/******************************************************************************
 *
 * File:        unixsignaling.h
 *
 * Description: This file contains the declaration of the UnixSignaling class.
 * The class handles Unix exit signals coming into the Window Manager and emits
 * a corresponding Qt signal to initialize a graceful shutdown of the Window
 * Manager and its clients.
 *
 * Based on the Qt example at: https://doc.qt.io/qt-6/unix-signals.html
 *
 * Copyright (C) 2023 CrossControl AB
 * All rights reserved.
 * Contact: CrossControl AB (info@crosscontrol.com)
 *
 * Code Template Revision: 1.2
 *
 ******************************************************************************/

/******************************************************************************
 * Nesting Header
 ******************************************************************************/
#ifndef UNIXSIGNALING_H
#define UNIXSIGNALING_H

/******************************************************************************
 * Include Files
 ******************************************************************************/
#include <QObject>
#include <QSocketNotifier>
#include <signal.h>

namespace WindowManager {
namespace Core {

    /******************************************************************************
     * Class Declaration
     ******************************************************************************/
    /**
     * @brief The UnixSignaling class takes care of handling unix signals such as
     * TERM, INT, HUP and QUIT and emits a corresponding Qt signal to be used in
     * Qt/QML
     */
    class UnixSignaling : public QObject
    {
        Q_OBJECT
    public:
        UnixSignaling(QObject *parent = 0);
        ~UnixSignaling();

        /**
         * @brief Handler for the INT signal
         * @param unused Unused int parameter to match sigaction handler
         */
        static void intSignalHandler(int unused);

        /**
         * @brief Handler for the TERM (terminate) signal
         * @param unused Unused int parameter to match sigaction handler
         */
        static void termSignalHandler(int unused);

        /**
         * @brief Handler for the HUP (hang-up) signal
         * @param unused Unused int parameter to match sigaction handler
         */
        static void hupSignalHandler(int unused);

        /**
         * @brief Handler for the QUIT signal
         * @param unused Unused int parameter to match sigaction handler
         */
        static void quitSignalHandler(int unused);

        /**
         * @brief Initializaiton function for the signaling management
         * @return error code, 0 if successful
         */
        int setupUnixSignalHandlers();

    signals:
        // Signal to emit when one of the four 'Quit' signals is received
        void unixSignalQuit();

    public slots:
        // Qt signal handlers
        void handleSigInt();
        void handleSigTerm();
        void handleSigHup();
        void handleSigQuit();

    private:
        static int sigIntFd[2];
        static int sigTermFd[2];
        static int sigHupFd[2];
        static int sigQuitFd[2];

        QSocketNotifier *snInt;
        QSocketNotifier *snTerm;
        QSocketNotifier *snHup;
        QSocketNotifier *snQuit;
    };

} // Core
} // WindowManager

#endif // UNIXSIGNALING_H
